<?php

// +----------------------------------------------------------------------+
// | ImageAlbum version 2.0                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002-2004 Joachim Fornallaz                            |
// +----------------------------------------------------------------------+
// | This source file is subject to the GPL license.                      |
// +----------------------------------------------------------------------+
// | Authors: Joachim Fornallaz <jf@omnis.ch>                             |
// +----------------------------------------------------------------------+

/* $Id: IACollection.php,v 1.14 2004/03/10 00:48:27 jfornall Exp $ */

if (file_exists("config.dev.inc.php")) {
	include_once("config.dev.inc.php");
} else {
	include_once("config.inc.php");
}

require_once("classes/IAHTML.php");
require_once("classes/IAImage.php");

class IACollection {

	var $numImages;
	var $offsetImages;
	var $offset;
	var $mode;
	var $usefolder;
	var $viewmode;
	var $commentsVisible;
	var $resizeUploadedImages;
	var $newsizeUploadedImages;
	var $limit;
	var $httpUpload;
	var $appearance;
	var $allowedImages;
	
	/* temp form data */
	var $formName;
	var $formEmail;
		
	/* db data */
	var $id;
	var $db;
	var $selectAll;
	var $domainID;
	var $userID;

	/* db fields */
	var $fields;
	var $folder;
	var $access;
	var $name;
	var $guestbook;
	var $rootPath;
	
	/* object references */
	var $refIAImage;
	
	/* constant definitions */
	var $ICON_LIMIT = 12;
	var $LIST_LIMIT = 8;
	var $RIGHTS = array("Everyone"=>"others","Nobody"=>"user","Users"=>"group");
	var $ICON_VIEW = "icon";
	var $LIST_VIEW = "list";

	/**
	 * constructor
	 *
	 * @param	&$domain	domain object reference
	 * @param	&$db		db object reference
	 * @param	$id			collecion id
	 * @param	$mode		'user' or 'group'
	 */
	function IACollection(&$domain,&$db,$id=NULL,$mode='view') {

		if (isset($_COOKIE['ImageAlbumIconLimit']))
			$this->ICON_LIMIT = $_COOKIE['ImageAlbumIconLimit'];
		if (isset($_COOKIE['ImageAlbumListLimit']))
			$this->LIST_LIMIT = $_COOKIE['ImageAlbumListLimit'];
				
		$this->id = $id;
		$this->db = &$db;
		$this->rootPath = $domain->rootPath;
		$this->domainID = $domain->id;
		$this->userID = $domain->refIAUser->id;
		$this->httpUpload = ($domain->getField("httpUpload") == "yes");
		$this->appearance = $domain->getField("htmlTemplate");
		$this->mode = $mode;
		$this->usefolder = $domain->cfg['datafolders'];
		$this->viewmode = $this->ICON_VIEW;
		$this->limit = $this->ICON_LIMIT;
		$this->offset = 0;
		$this->refIAImage = NULL;
		$this->allowedImages = array();
		$this->commentsVisible = true;
		$this->resizeUploadedImages = true;
		$this->newsizeUploadedImages = NULL;
		$this->access = "group";

		if ($id)
			$this->getDataFromDB();
	}

	/**
	 *
	 */
	function flow($action,$id=NULL) {

		switch ($action) {
			case "nextpage":
				if ($this->numImages > $this->limit)
					$this->offset += $this->limit;

				$maxoffset = (int)(ceil($this->numImages/$this->limit) - 1) * ($this->limit);

				if ($this->offset > $maxoffset)
					$this->offset = $maxoffset;
				break;
			case "prevpage":
				$this->offset -= $this->limit;
				$this->offset = max($this->offset,0);
				break;
			case "viewpage":
				$this->offset = ($id-1)*($this->limit);
				break;
			case "setimage":
				$this->refIAImage = new IAImage($this->db,$id);
				break;
			case "viewmode":
				if ($id == $this->LIST_VIEW) {
					$this->viewmode = $this->LIST_VIEW;
					$this->limit = $this->LIST_LIMIT;
				} else {
					$this->viewmode = $this->ICON_VIEW;
					$this->limit = $this->ICON_LIMIT;
				}
				break;
			case "seticonamount":
				$this->limit = $id;
				if ($this->viewmode == $this->ICON_VIEW) {
					setcookie("ImageAlbumIconLimit",$id,time()+(3600*24));
					$this->ICON_LIMIT = $id;
				} else {
					setcookie("ImageAlbumListLimit",$id,time()+(3600*24));
					$this->LIST_LIMIT = $id;
				}
				break;
		}
		
		if ($this->viewmode == $this->LIST_VIEW)
			$this->limit = $this->LIST_LIMIT;
		else
			$this->limit = $this->ICON_LIMIT;
	}
			
	/**
	 *
	 */
	function &getTemplateInstance($name,$removeEmptyBlocks=true,$formaction=null) {
		$tplFile = $this->appearance."/$name.template.html";
				
		if (!file_exists("./appearance/".$tplFile))
			$tplFile = "default/$name.template.html";

		$tpl = new HTML_Template_IT("./appearance");
		$tpl->loadTemplatefile($tplFile, true, $removeEmptyBlocks);
		
		
		if ($formaction) {
			$tpl->setVariable("PHP_SELF",$_SERVER['PHP_SELF']);
			$tpl->setVariable("ACTION",$formaction);
		}
		
		return $tpl;
	}
	
	/**
	 *
	 */
	function imageIsAllowed($img) {
		return (in_array($img,$this->allowedImages));
	}
	
	function commentsSetVisible($val) {
		$this->commentsVisible = ($val) ? true : false;
	}
	
	/**
	 *
	 */
	function setMode($mode) {
		$this->mode = $mode;
		
		if ($mode != "user") {
			$this->viewmode = $this->ICON_VIEW;
			$this->limit = $this->ICON_LIMIT;
		}
	}

	/**
	 *
	 */
	function setFolder($folder) {
		$this->folder = $folder;
		$this->name = $folder;
		$this->access = "group";
	}

	/**
	 *
	 */
	function getDataFromDB() {
		$sql = "SELECT * FROM `iacollection` WHERE `id` = $this->id";
		$res = $this->db->query($sql);
		$row = $res->fetchRow(DB_FETCHMODE_ASSOC);
		
		$this->fields = $row;
		$this->name = $row["name"];
		$this->folder = $row["folder"];
		$this->access = $row["access"];
		$this->guestbook = $row["guestbook"];
		$this->offset = 0;
						
		switch ($this->mode) {
			case "user":
				$this->selectAll = "SELECT * FROM `iaimage` WHERE `collection_id` = $this->id";
				break;
			case "group":
				$this->selectAll = "SELECT * FROM `iaimage` WHERE `collection_id` = $this->id AND access != 'user'";
				break;
			default:
				$this->selectAll = "SELECT * FROM `iaimage` WHERE `collection_id` = $this->id AND access != 'user' AND access != 'group'";
				break;
		}
	
		$res = $this->db->query($this->selectAll);
		$this->numImages = $res->numRows();
	}
	
	/**
	 *
	 */
	function getField($key) {
		return $this->fields[$key];
	}

	/**
	 *
	 */
	function update($form) {
		$sql = "UPDATE `iacollection` SET name='$form[name]',description='$form[description]',preview='$form[preview]',access='$form[access]',guestbook='$form[guestbook]' WHERE `id` = $this->id";
		$res = $this->db->query($sql);
				
		if (isset($form["sync"])) {
		
			$sql = "UPDATE `iaimage` SET `checked` = 0 WHERE `collection_id` = $this->id";
			$res = $this->db->query($sql);
			
			$pics = $this->scanDirectory($this->folder);
			foreach ($pics as $name=>$file) {
				$sql = "SELECT * FROM `iaimage` WHERE `collection_id` = $this->id AND path = '$this->folder/$file'";
				$res = $this->db->query($sql);
				
				if ($res->numRows() == 0) {
					$path = "$this->folder"."/$file";
					$sql = "INSERT INTO `iaimage` (name, path, collection_id, checked) VALUES ('$name', '$path', $this->id, 1)";
					$res = $this->db->query($sql);			
					$this->createThumbnail($path);
				} else {
					$row = $res->fetchRow(DB_FETCHMODE_ASSOC);
				
					$sql = "UPDATE `iaimage` SET `checked` = 1 WHERE `id` = ".$row["id"];
					$res = $this->db->query($sql);
				}	
			}
			
			$sql = "SELECT * FROM `iaimage` WHERE `collection_id` = $this->id AND `checked` = 0";
			$res = $this->db->query($sql);
		//	print_r($sql);
			
			while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$file = $row["path"];
				if (is_file($this->rootPath.".thumbnail/$file"))
					unlink($this->rootPath.".thumbnail/$file");

				$sql = "DELETE FROM `iaimage` WHERE id=".$row["id"];
				$res1 = $this->db->query($sql);
			}			
		}
		
		if (isset($form["saveusers"])) {
			$usrAll = $this->usersAll("id");
			$usrRel = $this->usersRel();
	
			if (!isset($form[users]))
				$form[users] = array();
	
			foreach ($usrAll as $usr) {
				if (in_array($usr,$form[users]) && !in_array($usr,$usrRel)) {
					$sql = "INSERT INTO `iausercollection` (IACollection_id, IAUser_id) VALUES ($this->id, $usr)";
					$res = $this->db->query($sql);
				} elseif (!in_array($usr,$form[users]) && in_array($usr,$usrRel)) {
					$sql = "DELETE FROM `iausercollection` WHERE IACollection_id = $this->id AND IAUser_id = $usr";
					$res = $this->db->query($sql);
				}
			}
		}

		$this->getDataFromDB();
	}
	
	/**
	 * id accessor
	 */
	function getID() {
		return $this->id;
	}

	/**
	 *
	 */
	function imageView($edit=false) {
		$prev = NULL;
		$next = NULL;

		$actionImage = $edit ? "admin.collection.imageedit" : "collection.imageview";
		$actionCollection = $edit ? "admin.collection.edit" : "collection.view";
		$this->allowedImages = array();

		$out = "\n<div class=\"navi01\">";
		
		if ($edit) {
			$out .= IAHTML::makeLink("Collections","admin.collection.list");
			$out .= " &gt; ";
		} else {
			$id = $this->refIAImage->getField("id");
			$visits = $this->refIAImage->getField("visits") + 1;
			$sql = "UPDATE `iaimage` SET visits=$visits WHERE `id` = $id";
			$res = $this->db->query($sql);
		}
		
		$out .= IAHTML::makeLink($this->name,$actionCollection);
		$out .= " &gt; ";
		$out .= $this->refIAImage->getField("name");
		$out .= " <span class=\"info\">(".$this->refIAImage->getField("visits")." visits)</span>";
		$out .= "</div>";

		if ($edit) {
			$out .= "\n<div class=\"navi02\">";
			
			$out .= IAHTML::formStart(array("id"=>$this->refIAImage->getField("id"),"action"=>"admin.collection.imagesave"));
			
			$out .= IAHTML::tableStart("imginfo",3);
			$out .= "<tr><td rowspan=\"4\" valign=\"top\" bgcolor=\"silver\"><img src=\"images/image.gif\" alt=\"Image\" width=\"16\" height=\"16\"></td>";
			$out .= IAHTML::formText("Name","name",$this->refIAImage->getField("name"));
			$out .= IAHTML::formPopup("Viewable for","access",$this->refIAImage->getField("access"),$this->RIGHTS)."</tr>";
			$out .= "<tr>".IAHTML::formInfo("Path",$this->refIAImage->getField("path"));
			$out .= "<td align=\"right\"><input type=\"checkbox\" name=\"form[resetcounter]\" value=\"true\"></td><td class=\"label\">Reset Counter</td></tr>";
		//	$out .= "<td></td><td class=\"label\">".IAHTML::makePopupLink("Rotate Image","popup.php",array("action"=>"admin.collection.imagerotate","image"=>$this->refIAImage->getField("path")))."</td></tr>";
			$out .= "<tr>".IAHTML::formTextarea("Description","description",$this->refIAImage->getField("description"),3)."</tr>";

			$out .= "<tr>".IAHTML::formButton("<input type=\"submit\" name=\"form[prev]\" value=\"&lt; Previous\" class=\"varbutton\"> <input type=\"submit\" value=\"Save\" class=\"varbutton\"> <input type=\"submit\" name=\"form[next]\" value=\"Next &gt;\" class=\"varbutton\">");
			$out .= IAHTML::tableEnd();
			
			$out .= IAHTML::formEnd();
	
			$out .= "</div>";
		}
		
		$this->imageNeighbors($this->refIAImage->getField("id"),$prev,$next);
		
	
		if ($prev) $out .= IAHTML::makeLink("<img src=\"images/prev.gif\" border=\"0\" width=\"25\" height=\"25\">",$actionImage,array("id"=>"$prev"));
		$out .= IAHTML::makeLink("<img src=\"images/parent.gif\" border=\"0\" width=\"25\" height=\"25\">",$actionCollection);
		if ($next) $out .= IAHTML::makeLink("<img src=\"images/next.gif\" border=\"0\" width=\"25\" height=\"25\">",$actionImage,array("id"=>"$next"));
	
		$desc = $this->refIAImage->getField("description");
		$desc = htmlentities($desc);
		$desc = nl2br($desc);
		$desc = str_replace("&gt;", ">", $desc);
		$desc = str_replace("&lt;", "<", $desc);
		$desc = str_replace("&amp;", "&", $desc);
	
		$style = ($edit) ? "navi02" : "";
		$out .= "<div align=\"center\">";
		$out .= IAHTML::tableStart($style,2,0);
		$out .= "<tr><td colspan=\"4\">".$this->getImageTag($this->refIAImage->getField("path"))."</td></tr>";

		if (!$edit) {
			$out .= "<tr><td colspan=\"4\" class=\"navi01\">".$desc."</td></tr>";
					
			if (($this->guestbook == "group" && $this->userID) || ($this->guestbook == "others"))
				$out .= $this->messagePane();
		} else {
			$out .= $this->messageList();
		}

		$out .= IAHTML::tableEnd();
		$out .= "</div>";
		
		$out .= "<img src=\"images/spacer.gif\" width=\"25\" height=\"25\">";
	
		return $out;
	}

	/**
	 *
	 */
	function messagePane() {
		
		if ($this->commentsVisible == false) {
			$out .= "<tr><td class=\"label\">".IAHTML::makeLink("<img border=\"0\" src=\"images/arrowclosed.gif\" alt=\"\" width=\"12\" height=\"12\">".$GLOBALS['strShowComments'],"collection.showcomments")."</td></tr>";
			return $out;
		} else {
			$out .= "<tr><td class=\"label\">".IAHTML::makeLink("<img border=\"0\" src=\"images/arrowopen.gif\" alt=\"\" width=\"12\" height=\"12\">".$GLOBALS['strHideComments'],"collection.hidecomments")."</td></tr>";
		}

		$imageID = $this->refIAImage->getField("id");
	
		$sql = "SELECT * FROM `iacomment` WHERE `image_id` = $imageID ORDER BY timestamp DESC LIMIT 10";
		$res = $this->db->query($sql);
		$i = 0;
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
		
			$datestr = $this->coolDate($row['timestamp']);
			$sender = (strlen($row['name'])>0) ? htmlentities($row['name']) : "Anonymous";
			
			if (strlen($row['name'])>0)
				$from = htmlentities($row['name']);
			
			if (strlen($row['email'])>0) {
				$from = "<a href=\"mailto:".$row['email']."\">$sender</a>";
			} else {
				$from = $sender;
			}

			$color = ($i % 2 == 0) ? "#e6f2f2" : "#fefefe";
			$i++;
			
			$out .= "<tr bgcolor=\"$color\"><td class=\"label\">$from<br><span class=\"info\">$datestr</span></td><td valign=\"top\" colspan=\"3\">".htmlentities($row['comment'])."</td></tr>";
		}
	
		$out .= IAHTML::formStart(array("form[user_id]"=>$this->userID,"form[image_id]"=>$imageID,"action"=>"collection.addcomment"));
		$out .= "<tr bgcolor=\"#dddddd\">".IAHTML::formText($GLOBALS['strName'],"name",$this->formName).IAHTML::formText($GLOBALS['strEmail'],"email",$this->formEmail)."</tr>";
		$out .= "<tr bgcolor=\"#dddddd\">".IAHTML::formText($GLOBALS['strYourComment'],"comment","")."<td>&nbsp;</td><td align=\"right\"><input type=\"submit\" value=\"".$GLOBALS['strSubmit']."\" class=\"button\"></td></tr>";
		$out .= IAHTML::formEnd();
		return $out;
	}

	/**
	 *
	 */
	function messageList() {

		$imageID = $this->refIAImage->getField("id");
	
		$sql = "SELECT * FROM `iacomment` WHERE image_id = $imageID ORDER BY timestamp DESC";
		$res = $this->db->query($sql);

		$out .= "<tr><td colspan=\"4\" class=\"navi01\">Comments</td></tr>";
		$out .= IAHTML::formStart(array("action"=>"admin.collection.deletecomment","id"=>$imageID));
		$out .= "<tr><td colspan=\"2\"><select name=\"form[comments][]\" size=\"6\" multiple>";
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$sender = (strlen($row['name'])>0) ? htmlentities($row['name']) : "Anonymous";
			$datestr = $this->coolDate($row['timestamp']);

			if (strlen($row['comment'])>40) {
				$rest = substr($row['comment'], 0, 40);
				$rest = htmlentities($rest)."...";
			} else {
				$rest = htmlentities($row['comment']);
			}

			$out .= "\n<option value=\"".$row['id']."\">$datestr - $sender - $rest</option>";
		}
		
		$out .= "</select></td>".IAHTML::formCheckBox("Delete All","deleteall","true")."</tr>";
		$out .= "<tr>".IAHTML::formButton("<input type=\"submit\" value=\"Delete\" class=\"button\">")."</tr>";
		$out .= IAHTML::formEnd();
	
		return $out;
	}
	
	/**
	 *
	 */
	function coolDate($sqlDate) {
		$ts = strtotime($sqlDate);
		
		$now = date("m.d.y");
		$then = date("m.d.y",$ts);
		
		if ($now == $then)
			$ret = "Today, ".date("g:i a",$ts);
		else
			$ret = date("F j, Y, g:i a",$ts);
			
		return $ret;
	}

	/**
	 * 
	 */
	function addComment($form) {
	
		if (isset($form['image_id']) && strlen($form['comment'])>2) {
				
			$image_id = $form['image_id'];
			$user_id = $form['user_id'];
			$comment = strip_tags($form['comment']);
			$name = strip_tags($form['name']);
			$email = strip_tags($form['email']);
			
			$this->formName = $name;
			$this->formEmail = $email;
		
			$sql = "INSERT INTO `iacomment` (user_id, image_id, timestamp, comment, name, email) VALUES ('$user_id', '$image_id', now(), '$comment', '$name', '$email')";
			$res = $this->db->query($sql);
		}
	}

	/**
	 *
	 */
	function deleteComments($id,$form) {
	
		$ids = $form['comments'];
	
		if ($form['deleteall'] == true && $id > 0) {
			$sql = "DELETE FROM `iacomment` WHERE image_id = $id";			
			$res = $this->db->query($sql);
		} elseif (is_array($ids)) {
			foreach ($ids as $cid) {
				$sql = "DELETE FROM `iacomment` WHERE `id` = $cid";			
				$res = $this->db->query($sql);
			}
		}
	}
	
	/**
	 *
	 */
	function imageSave($id,$form) {
		$sql = "UPDATE `iaimage` SET name='$form[name]',description='$form[description]',access='$form[access]' WHERE `id` = $id";
		$res = $this->db->query($sql);
		
		if ($form['resetcounter']) {
			$sql = "UPDATE `iaimage` SET visits=0 WHERE `id` = $id";
			$res = $this->db->query($sql);
		}
	}

	/**
	 *
	 */
	function imageListSave($ids,$form) {
		foreach ($ids as $i=>$id) {
			$imageName = $form["name$i"];
			$imageAccess = $form["access$i"];
			
			$sql = "UPDATE `iaimage` SET name='$imageName',access='$imageAccess' WHERE `id` = $id";
			$res = $this->db->query($sql);
		}
	}

	/**
	 *
	 */
	function imageNeighbors($id,&$prev,&$next) {
		$sql = $this->selectAll." ORDER BY path";
		$res = $this->db->query($sql);
		$ids = array();
		$pos = -1;
		$i = 0;

		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$ids[] = $row["id"];
			if ($row["id"] == $id)
				$pos = $i;
				
			$i++;
		}
		
		$prev = $ids[$pos-1];
		$next = $ids[$pos+1];
		
		$this->offset = (int)($pos/$this->limit) * ($this->limit);
	}

	/**
	 *
	 */
	function dataEdit($showUserList=false) {
		
		$previewOptions = array("Image &amp; Text"=>"image_and_text", "Image Only"=>"image", "Text Only"=>"text", "None"=>"no");	

		$tpl = &$this->getTemplateInstance("admin.collection",true,"admin.collection.update");
		$tpl->setVariable("ID",$this->id);
		$tpl->setVariable("name",$this->getField("name"));
		$tpl->setVariable("access",IAHTML::getFormPopup("access",$this->getField("access"),$this->RIGHTS));
		$tpl->setVariable("folder",$this->getField("folder"));
		$tpl->setVariable("description",$this->getField("description"));
		$tpl->setVariable("guestbook",IAHTML::getFormPopup("guestbook",$this->getField("guestbook"),$this->RIGHTS));
		$tpl->setVariable("preview",IAHTML::getFormPopup("preview",$this->getField("preview"),$previewOptions));
	
		if ($this->httpUpload == false) {
			$tpl->setCurrentBlock("no_http");
			$tpl->setVariable("SYNC",IAHTML::getFormCheckbox("sync","true"));
			$tpl->parseCurrentBlock("no_http");
		}
		
		if ($showUserList) {
			$tpl->setCurrentBlock("userlist_visible");
			$tpl->setVariable("HideUserLink", $_SERVER['PHP_SELF']."?action=admin.collection.edit");

			$relUsers = $this->usersRel();
			$allUsers = $this->usersAll();
			
			$i = 0;
			foreach ($allUsers as $oneUser) {
				$tpl->setCurrentBlock("useritem");
				if ($i % 3 == 0)
					$tpl->setVariable("TR_OPEN","<tr>");
				$tpl->setVariable("user_checkbox",IAHTML::getFormCheckboxUN("form[users][]",$oneUser["id"],in_array($oneUser["id"],$relUsers)));
				$tpl->setVariable("user_name",$oneUser["name"]);

				if ($i % 3 == 2) 
					$tpl->setVariable("TR_CLOSE","</tr>");

				$tpl->parseCurrentBlock("useritem");
				$i++;
			}
			if ($i % 3 != 0) 
				$tpl->setVariable("TR_CLOSE2","</tr>");
				
			$tpl->parseCurrentBlock("userlist_visible");
		} else {
			$tpl->setCurrentBlock("userlist_hidden");
			$tpl->setVariable("ShowUserLink", $_SERVER['PHP_SELF']."?action=admin.collection.userlist");
			$tpl->parseCurrentBlock("userlist_hidden");
		}
		
		return $tpl->get();
	}

	/**
	 *
	 */
	function userList($edit=true) {
		$out = "\n<div class=\"navi01\">";

		if ($edit) {
			$out .= IAHTML::makeLink("Collections","admin.collection.list");
			$out .= " &gt; ";
		}
		
		$out .= $this->name;
		$out .= "</div>";

		if ($this->id == NULL)
			return;

		if ($edit) {
			$out .= $this->dataEdit(true);
		}

		return $out;
	}


	/**
	 *
	 */
	function imageList($edit=false) {
		
		switch($this->mode) {
			case "user":	$action = "edit";	$edit = true;	break;
			case "group":	$action = "view";	break;
			default: 		$action = "view";	break;
		}

		$actionImage = $edit ? "admin.collection.imageedit" : "collection.imageview";
		$actionCollection = $edit ? "admin.collection.edit" : "collection.view";
		$this->allowedImages = array();

		$out = "\n<div class=\"navi01\">";

		if ($edit) {
			$out .= IAHTML::makeLink("Collections","admin.collection.list");
			$out .= " &gt; ";
		}
		
		$out .= $this->name;
		$out .= "</div>";

		if ($this->id == NULL)
			return;

		if ($edit) {
			$out .= $this->dataEdit();
			$out .= "<table align=\"right\" border=\"0\"><tr><td>".$this->viewModeChooser($actionCollection)."</td></tr></table>";
		}

		$sql = $this->selectAll." ORDER BY path LIMIT $this->offset,$this->limit";
		$res = $this->db->query($sql);
		$out .= $this->listNavigation($actionCollection,$res->numRows());
		
		// Upload form
		if ($edit && $this->httpUpload == true) {
			$out .= $this->uploadForm();
		}
		
		// List view
		if ($edit && $this->viewmode == $this->LIST_VIEW)
			$out .= $this->imageListView($edit,$res);
		else
			$out .= $this->imageIconView($edit,$res,$actionImage);

	//	print_r($this->allowedImages);

		return $out;
	}

	/**
	 * generate html tag for thumbnail image display
	 */
	function getImageTag($path) {
	//	$out = "<img border=\"0\" width=\"128\" height=\"128\" src=\"".$this->rootPath.".thumbnail/".$path."\">";
		$out = "<img border=\"0\" src=\"stream.php?path=".urlencode($path)."\">";
		$this->allowedImages[] = $path;
		return $out;
	}

	/**
	 * generate html tag for image display
	 */
	function getThumbnailImageTag($path) {
	//	$out = "<img border=\"0\" width=\"128\" height=\"128\" src=\"".$this->rootPath.".thumbnail/".$path."\">";
		$out = "<img border=\"0\" width=\"128\" height=\"128\" src=\"stream.php?view=thumbnail&amp;path=".urlencode($path)."\">";
	//	$out .= "<a href=\"stream.php?view=thumbnail&amp;path=".urlencode($path)."\">Image</a>";
		$this->allowedImages[] = $path;
		return $out;
	}

	/**
	 * generate image browsing view in icon mode
	 */
	function imageIconView($edit,$res,$action) {
		
		$tpl = &$this->getTemplateInstance("thumbnail");
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
					
			if ($edit && $this->httpUpload == true) {
				$tpl->setCurrentBlock("editing");
				$ac = array_search($row["access"],$this->RIGHTS);
				$tpl->setVariable("DELETELINK",IAHTML::deleteLinkLocation("admin.collection.imagedelete",array("id"=>$row["id"])));
				$tpl->setVariable("ACCESS",$ac);
				$tpl->parseCurrentBlock("editing");
			}
			$tpl->setCurrentBlock("thumbnail");
			$tpl->setVariable("IMAGE",$this->getThumbnailImageTag($row["path"]));
			$tpl->setVariable("IMAGELINK",IAHTML::linkLocation($action,array("id"=>$row["id"])));
			$tpl->setVariable("LABEL",$row["name"]);
			$tpl->parseCurrentBlock("thumbnail");
		}
		
		return $tpl->get();
	}


	/**
	 * generate upload form (used in http upload mode)
	 */
	function uploadForm() {

		$tpl = &$this->getTemplateInstance("admin.uploadform",true,"admin.collection.imageupload");

		$checked = ($this->resizeUploadedImages == true) ? "checked" : "";
		$newsize = ($this->newsizeUploadedImages == NULL) ? 640 : $this->newsizeUploadedImages;
	
		$tpl->setVariable("LIMMITSIZECHECKED",$checked);
		$tpl->setVariable("NEWSIZE",$newsize);
		return $tpl->get();
	}
	
	/**
	 * import image into db and disk ($this->httpUpload == true)
	 */
	function importImage($form) {
		$data = $_FILES['file01'];
				
		if (!$data)
			return false;
		
		$fname = $data['name'];
		$ftemp = $data['tmp_name'];
		$typeOK = (($data['type'] == "image/jpeg") || ($data['type'] == "image/pjpeg") || ($data['type'] == "image/png"));
		$sizeOK = ($data['size'] < 1000000);
				
		$destpath = ($this->usefolder) ? $this->rootPath."/".$this->folder."/".$fname : $this->rootPath."/".$fname;
		
		if ($typeOK && $sizeOK && move_uploaded_file($ftemp,$destpath)) {
			$domainID = $this->domainID;
			chmod($destpath, 0755);

			$path = ($this->usefolder) ? $this->folder."/".$fname : $fname;
			$sql = "INSERT INTO `iaimage` (domain_id, name, path, collection_id, access) VALUES ($domainID, '$fname', '$path', $this->id, '$this->access')";
			$res = $this->db->query($sql);

			$pInfo = pathinfo($fname);
			$ext = strtolower($pInfo["extension"]);

			if (!$this->usefolder && strlen($ext)) {
				$sql = "SELECT LAST_INSERT_ID()";
				$res = $this->db->query($sql);
				$row = $res->fetchRow();
				$lastID = $row[0];
				$path = "image$lastID.$ext";
				$sql = "UPDATE `iaimage` SET path='$path' WHERE `id` = $lastID";
				$res = $this->db->query($sql);
				
				$ok = rename($destpath,$this->rootPath."/".$path);
				chmod($this->rootPath."/".$path, 0777);
				assert($ok == true);
			}

			if (is_numeric($form['sizelimit']) && $form['doresize']) {
				$this->resizeUploadedImages = true;
				$this->newsizeUploadedImages = $form['sizelimit'];
				$this->resizeImage($path,$form['sizelimit']);
			} else {
				$this->resizeUploadedImages = false;
			}

			$this->createThumbnail($path);
			$this->getDataFromDB();
			return true;
		} else {
			return false;
		}
	}

	/**
	 * delete image from db and disk ($this->httpUpload == true)
	 */
	function deleteImage($id) {
		$sql = "SELECT `path` FROM `iaimage` WHERE `id` = $id";
		$res = $this->db->query($sql);
		$row = $res->fetchRow();
		$path = $row[0];
		
		if ($this->imageIsAllowed($path)) {
			$sql = "DELETE FROM `iacollection` WHERE `image_id` = $id";
			$res = $this->db->query($sql);
	
			$sql = "DELETE FROM `iaimage` WHERE `id` = $id";
			$res = $this->db->query($sql);
			
			$sql = "DELETE FROM `iacomment` WHERE image_id = $id";			
			$res = $this->db->query($sql);

			if (is_file($this->rootPath.".thumbnail/".$path))
				unlink($this->rootPath.".thumbnail/".$path);

			if (is_file($this->rootPath."/".$path))
				unlink($this->rootPath."/".$path);
				
			$this->getDataFromDB();
		}
	}

	/**
	 * generate image browsing and editing view in list mode
	 */
	function imageListView($edit,$res) {
		$actionImage = "admin.collection.imageedit";
		$imageCount = 0;
		$out = "";
		$out .= "\n<div align=\"center\">";
		$out .= IAHTML::formStart(array("action"=>"admin.collection.imagelistsave"));
		$out .= "<input type=\"submit\" name=\"form[next]\" value=\"Save\" class=\"hidden\">";

		$ptable = new IATable("list");
		$ptable->setNumCellsInRow(4);
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
		
			$imageRef = new IAImage($this->db,$row["id"]);
			$content = IAHTML::makeLink($this->getThumbnailImageTag($row["path"]),$actionImage,array("id"=>$row["id"]));
			$ptable->addCell($content,"navi02");

			$content = IAHTML::tableStart("imginfo",3);
			$content .= "<tr>".IAHTML::formText("Name","name$imageCount",$imageRef->getField("name"))."</tr>";
			$content .= "<tr>".IAHTML::formPopup("Access","access$imageCount",$imageRef->getField("access"),$this->RIGHTS)."</tr>";
			$content .= "<tr>".IAHTML::formInfo("Path",$imageRef->getField("path"))."</tr>";
			$content .= "<tr><input type=\"hidden\" name=\"id[$imageCount]\" value=\"".$row["id"]."\"><td></td><td class=\"label\">";
			
			if ($edit && $this->httpUpload == true) {
				$ac = array_search($row["access"],$this->RIGHTS);
				$content .= IAHTML::makeLink("<img src=\"images/delete.gif\" alt=\"rm\" border=\"0\" width=\"9\" height=\"9\" align=\"right\">","admin.collection.imagedelete",array("id"=>$row["id"])); 
			}
			
			$content .= "</td></tr>";
			$content .= IAHTML::tableEnd();
			$ptable->addCell($content,"navi02");
			
			$imageCount++;
		}

		$content = "<input type=\"submit\" name=\"form[prev]\" value=\"&lt; Previous\" class=\"button\"> <input type=\"submit\" value=\"Save\" class=\"button\"> <input type=\"submit\" name=\"form[next]\" value=\"Next &gt;\" class=\"button\">";
		$ptable->addCellLine($content,"navi03");
		$out .= $ptable->toHTML();
		$out .= IAHTML::formEnd();
		$out .= "\n</div>";
		return $out;
	}

	/**
	 * generate navigation arrows for image list
	 */
	function listNavigation($action,$nr) {
		$out = "";
		$numbers = array(8,12,16,20,24,28);
	
		if ($nr < $this->numImages) {
			$tpl = &$this->getTemplateInstance("imagenavigation",true,$action);
			$tpl->setVariable("SUBACTION","seticonamount");

			foreach ($numbers as $number) {
				$tpl->setCurrentBlock("amountoption");
				$tpl->setVariable("NUMBER",$number);
				
				if ($number == $this->limit) $tpl->setVariable("SELECTED","selected");
				
				$tpl->parseCurrentBlock("amountoption");
			}

			$start = $this->offset + 1;
			$stop = $this->offset + $this->limit;
			$page = (int)($this->offset / $this->limit) + 1;
			$pages = (int) ceil($this->numImages / $this->limit);
			
			$tpl->setVariable("PREVLINK",IAHTML::linkLocation($action,array("sub"=>"prevpage","page"=>($page-1))));
			$tpl->setVariable("NEXTLINK",IAHTML::linkLocation($action,array("sub"=>"nextpage","page"=>($page+1))));
			$tpl->setVariable("PAGEINFO","$GLOBALS[strPage] $page $GLOBALS[strOf] $pages");
			$tpl->setVariable("IMAGEINFO", "$this->numImages $GLOBALS[strImagesAvailable]");
			$tpl->setVariable("GOTO",$GLOBALS[strGoto]);
			for ($i=1; $i<=$pages; $i++) {
				if ($page == $i) {
					$tpl->setCurrentBlock("thispage");
					$tpl->setVariable("THISPAGENUMBER", $page);
					$tpl->parseCurrentBlock("thispage");
				} else {
					$tpl->setCurrentBlock("otherpage");
					$tpl->setVariable("PAGENUMBER", $i);
					$tpl->setVariable("PAGELINK", IAHTML::linkLocation($action,array("sub"=>"viewpage","sval"=>$i)));
					$tpl->parseCurrentBlock("otherpage");
				}
			}
			
			$out = $tpl->get();
		}
		return $out;
	}

	/**
	 * generate navigation arrows for image list
	 */
	function viewModeChooser($action) {
		$out = "";
		if ($this->viewmode == $this->ICON_VIEW)
			$out .= "<img src=\"images/viewModeIconSelected.gif\" width=\"20\" height=\"25\" border=\"0\">".IAHTML::makeLink("<img src=\"images/viewModeList.gif\" width=\"20\" height=\"25\" border=\"0\">",$action,array("sub"=>"viewmode","sval"=>$this->LIST_VIEW));
		else
			$out .= IAHTML::makeLink("<img src=\"images/viewModeIcon.gif\" width=\"20\" height=\"25\" border=\"0\">",$action,array("sub"=>"viewmode","sval"=>$this->ICON_VIEW))."<img src=\"images/viewModeListSelected.gif\" width=\"20\" height=\"25\" border=\"0\">";
			
		return $out;			
	}


	/**
	 * get array of all related users
	 */
	function usersRel() {
		$relUsers = array();

		if ($this->id==NULL)
			return $relUsers;
		
		$sql = "SELECT `iauser`.`id` FROM `iacollection`,IAUser,`iausercollection` WHERE `iacollection`.`id` = `iausercollection`.IACollection_id AND `iauser`.`id` = `iausercollection`.IAUser_id AND `iacollection`.`id` = $this->id";
		$res = $this->db->query($sql);
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$relUsers[] = $row["id"];
		}
		
		return $relUsers;
	}

	/**
	 * get array of all users
	 */
	function usersAll($field=NULL) {
		$sql = "SELECT * FROM `iauser` WHERE `domain_id` = $this->domainID";
		$res = $this->db->query($sql);
		$all = array();

		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))
			$all[] = ($field) ? $row[$field] : $row;
		
		return $all;
	}

	/**
	 * 
	 */
	function scanDirectory($dir) {
		$pictures = array();
		$d = dir($this->rootPath."/$dir");
		$this->folder = $dir;
		
		while($entry=$d->read()) {
			if (eregi("(.*)\.(jpg|jpeg|gif|png)", $entry, $regs)) {
				$pictures[$regs[1]] = $entry;
			}
		}
		return $pictures;
	}

	/**
	 * 
	 */
	function importOptions() {
		$out = "\n<div class=\"navi01\">";

		$out .= IAHTML::makeLink("Collections","admin.collection.list");
		$out .= " &gt; ".$this->name."</div>";
		
		
		$out .= IAHTML::formStart(array("id"=>$this->folder,"action"=>"admin.collection.import"));
			
		$out .= "\n<div class=\"navi02\">";
		$out .= "";
		$out .= IAHTML::tableStart("colinfo",3);
		$out .= "<tr><td rowspan=\"5\" valign=\"top\" bgcolor=\"silver\"><img src=\"images/album.gif\" alt=\"Image\" width=\"16\" height=\"16\"></td>";
		$out .= IAHTML::formText("Name","name",$this->name);
		$out .= IAHTML::formPopup("Collection Access","colaccess",$this->access,$this->RIGHTS)."</tr>";
		$out .= "<tr>".IAHTML::formInfo("Path",$this->folder);
		$out .= IAHTML::formPopup("Image Access","imgaccess",$this->access,$this->RIGHTS)."</tr>";
	//	$out .= "<tr>".IAHTML::formCheckBox("Resize Images to 600 Pixel","resize","true")."</tr>";
		$out .= "<tr><td></td><td class=\"label\">".IAHTML::makePopupLink("Resize Images","popup.php",array("action"=>"admin.collection.resize","collection"=>$this->folder))."</td></tr>";
		$out .= "<tr>".IAHTML::formButton("<input type=\"submit\" value=\"Import\" class=\"button\"> <input type=\"reset\" value=\"Reset\" class=\"button\">")."</tr>";

		$out .= IAHTML::tableEnd();
		$out .= "\n</div>";
		
		$out .= IAHTML::formEnd();
		
		return $out;
	}


	/**
	 * 
	 */
	function newFolderOptions() {
		$out = "\n<div class=\"navi01\">";

		$out .= IAHTML::makeLink("Collections","admin.collection.list");
		$out .= " &gt; ".$this->name."</div>";
		
		
		$out .= IAHTML::formStart(array("id"=>$this->folder,"action"=>"admin.collection.insert"));
			
		$out .= "\n<div class=\"navi02\">";
		$out .= "";
		$out .= IAHTML::tableStart("colinfo",3);
		$out .= "<tr><td rowspan=\"5\" valign=\"top\" bgcolor=\"silver\"><img src=\"images/album.gif\" alt=\"Image\" width=\"16\" height=\"16\"></td>";
		$out .= IAHTML::formText("Name","name",$this->name);
		$out .= IAHTML::formPopup("Collection Access","colaccess",$this->access,$this->RIGHTS)."</tr>";
		
		if ($this->usefolder)
			$out .= "<tr>".IAHTML::formText("Path","folder",$this->folder)."</tr>";
		$out .= "<tr>".IAHTML::formButton("<input type=\"submit\" value=\"Add\" class=\"button\"> <input type=\"reset\" value=\"Reset\" class=\"button\">")."</tr>";

		$out .= IAHTML::tableEnd();
		$out .= "\n</div>";
		
		$out .= IAHTML::formEnd();
		
		return $out;
	}


	/**
	 * 
	 */
	function importToDB($dir,$form) {
		$this->folder = $dir;
		$domainID = $this->domainID;
			
		$sql = "INSERT INTO `iacollection` (domain_id, name, folder, access) VALUES ($domainID, '$form[name]', '$dir', '$form[colaccess]')";
		$res = $this->db->query($sql);

		$sql = "SELECT LAST_INSERT_ID()";
		$res = $this->db->query($sql);
		$row = $res->fetchRow();
		
		$this->id = $row[0];
		$succ = false;
		
		if (!file_exists($this->rootPath.".thumbnail/$this->folder")) {
			$succ = mkdir($this->rootPath.".thumbnail/$this->folder",0777);
			if (!$succ) echo "Could not create the thumbnail directory $this->folder!";
		}
		
		$pics = $this->scanDirectory($dir);
		foreach ($pics as $name=>$file) {
			$path = "$this->folder"."/$file";
			$sql = "INSERT INTO `iaimage` (domain_id, name, path, collection_id, access) VALUES ($domainID, '$name', '$path', $this->id, '$form[imgaccess]')";
			$res = $this->db->query($sql);
			
			$this->createThumbnail($path);
			
			if ($form['resize'] == true)
				$this->resizeTo600($path);
		}
		
		$this->setMode("user");
		$this->getDataFromDB();
	}

	
	/**
	 * 
	 */
	function insert($form) {
		$domainID = $this->domainID;
		$this->folder = $form['folder'];
		
		if (strlen($this->folder) < 1 && $this->usefolder)
			return -1;
			
		$sql = "INSERT INTO `iacollection` (domain_id, name, folder, access) VALUES ($domainID, '$form[name]', '$form[folder]', '$form[colaccess]')";
		$res = $this->db->query($sql);

		$sql = "SELECT LAST_INSERT_ID()";
		$res = $this->db->query($sql);
		$row = $res->fetchRow();
		
		$this->id = $row[0];
		$succ = false;

		if ($this->usefolder) {
			if (!file_exists($this->rootPath."/$this->folder")) {
				$succ = mkdir($this->rootPath."/$this->folder",0777);
				if (!$succ) echo "Could not create the image directory $this->folder!";
			}
			
			if (!file_exists($this->rootPath.".thumbnail/$this->folder")) {
				$succ = mkdir($this->rootPath.".thumbnail/$this->folder",0777);
				if (!$succ) echo "Could not create the thumbnail directory $this->folder!";
			}
		}
		
		$this->setMode("user");
		$this->getDataFromDB();
	}
	
	/**
	 *
	 */
	function resizeImage($path,$size) {
		assert(function_exists("imagecopyresampled") == true);
		$path = $this->rootPath."/".$path;
			
		$pInfo = pathinfo($path);
		$ext = $pInfo["extension"];
			
		if ($ext == "jpeg" || $ext == "jpg") 
			$im = @imagecreatefromjpeg($path);
		elseif ($ext == "png")
			$im = @imagecreatefrompng($path);
		else
			return;	
			
		$sx = imagesx($im);
		$sy = imagesy($im);
		
		if(max($sx,$sy) <= $size)
			return;
		
		if ($sx >= $sy) {
			$factor = (float)$sx / (float)$size;
			$newWidth = $size;
			$newHeight = (int)((float)$sy/$factor);
		} else {
			$factor = (float)$sy / (float)$size;
			$newWidth = (int)((float)$sx/$factor);
			$newHeight = $size;
		}
		
		$rim = @imagecreatetruecolor($newWidth, $newHeight);
		
		imagecopyresampled($rim, $im, 0, 0, 0, 0, $newWidth, $newHeight, $sx, $sy);
		
		if ($ext == "jpeg" || $ext == "jpg") 
			imagejpeg($rim, $path);
		else
			imagepng($rim, $path);
	}


	/**
	 * 
	 */
	function createThumbnail($path) {	
		
		if(function_exists("imagecopyresampled") == true) {		
			$thumbnail = "images/thumbnailBack.jpg";
			$tpath = $this->rootPath.".thumbnail/".$path;
			$path = $this->rootPath."/".$path;
					
			$pInfo = pathinfo($path);
			$tpInfo = pathinfo($tpath);
			$ext = strtolower($pInfo["extension"]);
			
			if ($ext == "jpeg" || $ext == "jpg") 
				$sim = imagecreatefromjpeg($path);
			elseif ($ext == "png")
				$sim = imagecreatefrompng($path);
			else
				return;

			$dim = imagecreatefromjpeg($thumbnail);
			
			$sx = imagesx($sim);
			$sy = imagesy($sim);
			$tx = imagesx($dim);
			$ty = imagesy($dim);
			
			if ($sx >= $sy) {	// landscape
				$factor = (float)$sx / ((float)$tx - 2);
				$dstX = 1;
				$dstH = (int)((float)$sy / $factor);
				$dstY = ($tx - $dstH) / 2 + 1;
				$dstW = $tx - 2;
			} else {
				$factor = (float)$sy / ((float)$ty - 2);
				$dstY = 1;
				$dstW = (int)((float)$sx / $factor);
				$dstX = ($ty - $dstW) / 2 + 1;
				$dstH = $ty - 2;			
			}
			
			imagecopyresampled($dim, $sim, $dstX, $dstY, 0, 0, $dstW, $dstH, $sx, $sy);
			$newThumbnailImage = $tpInfo["dirname"]."/".$pInfo["basename"];

			if ($ext == "jpeg" || $ext == "jpg") 
				imagejpeg($dim, $newThumbnailImage,90);
			else
				imagepng($dim, $newThumbnailImage);
				
			chmod($newThumbnailImage, 0777);
		} else {
			$cmd = $GLOBALS["binarypath"]."/montage -quality 85 -background gray  -bordercolor gray -borderwidth 1x1 -geometry 128x128 ".$this->rootPath."/$path ".$this->rootPath.".thumbnail/$path";
		//	echo "$cmd<br>";
			passthru($cmd);
		}
	}

	/**
	 * 
	 */
	function resizeTo600($path) {
		$cmd = $GLOBALS["binarypath"]."/mogrify -quality 85 -geometry 600x600 ".$this->rootPath."/$path";
	//	echo "$cmd<br>";
		passthru($cmd);
	}

	/**
	 * 
	 */
	function deleteFromDB() {
		if ($this->id == NULL)
			return;
			
		$sql = "SELECT `id`,`path` FROM `iaimage` WHERE `collection_id` = $this->id";
		$res = $this->db->query($sql);
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$imgpath = $this->rootPath.".thumbnail/".$row['path'];
			if (is_file($imgpath) && !($this->usefolder))
				unlink($imgpath);

			$sql = "DELETE FROM `iacomment` WHERE `image_id` = ".$row['id'];			
			$rs2 = $this->db->query($sql);
		}
		
		$sql = "DELETE FROM `iausercollection` WHERE `IACollection_id` = $this->id";
		$res = $this->db->query($sql);

		$sql = "DELETE FROM `iacollection` WHERE `id` = $this->id";
		$res = $this->db->query($sql);

		$sql = "DELETE FROM `iaimage` WHERE `collection_id` = $this->id";
		$res = $this->db->query($sql);
						
		if ($this->usefolder && file_exists($this->rootPath.".thumbnail/$this->folder")) {
			$d = dir($this->rootPath.".thumbnail/$this->folder");
	
			while($entry=$d->read()) {
				if (is_file($this->rootPath.".thumbnail/$this->folder/$entry"))
					unlink($this->rootPath.".thumbnail/$this->folder/$entry");
			}
			rmdir($this->rootPath.".thumbnail/$this->folder");
		}
	}
}

?>
