<?php

// +----------------------------------------------------------------------+
// | ImageAlbum version 2.0                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002-2004 Joachim Fornallaz                            |
// +----------------------------------------------------------------------+
// | This source file is subject to the GPL license.                      |
// +----------------------------------------------------------------------+
// | Authors: Joachim Fornallaz <jf@omnis.ch>                             |
// +----------------------------------------------------------------------+

/* $Id: IADomain.php,v 1.15 2004/03/10 00:50:27 jfornall Exp $ */

class IADomain {

	// Variables

	var $id;
	var $db;
	var $cfg;

	var $refIAUser = NULL;
	var $refIAEditUser = NULL;
	var $refIACollection = NULL;

	var $alert;
	var $fields; 
	var $screen;
	var $rootPath = "data";
	var $forceLogin;
	var $allowedImages;

	/**
	 * Contructor
	 *
	 * @param	$db
	 * @param	$id
	 */
	function IADomain(&$db,&$cfg,$id=NULL) {
		$this->id = $id;
		$this->db = &$db;
		$this->cfg = &$cfg;
		$this->allowedImages = array();

		$this->refIAUser = new IAUser($this->id,$this->db);
		$this->forceLogin = false;

		if ($id) {
			$this->getDataFromDB();	
		} else {
			$this->fields = array("name"=>"[new domain]","guestonly"=>"no");
		}
	}

	/**
	 * setter method
	 *
	 * @param	$thescreen
	 */
	function setScreen($thescreen) {
		$this->screen = $thescreen;
	}

	/**
	 * setter method
	 *
	 * @param	$theAlert
	 */
	function setAlert($theAlert) {
		$this->alert = $theAlert;
	}

	/**
	 * setter method
	 *
	 * @param	$path		
	 */
	function setRootPath($path) {
		$this->rootPath = $this->getDataPath($path);
	}
	
	/**
	 *
	 */
	function flow($action,$id,$sub,$sval,$form) {
		$actionArray = explode(".",$action);
		$this->setScreen("");		

		$viewmode =	$this->refIAUser->isAuthenticated() ? "group" : "others";

		switch ($actionArray[0]) {
			case "admin":
				$action = ($this->refIAUser->isAdmin()) ? $action : "notpermitted";
				break;
		}

		switch ($action) {
			/* user admin */
			case "admin.user.list":
				$this->setScreen("userlist");		
				break;
			case "admin.user.update":
				$this->setScreen("userlist");
				$this->updateUser($id,$form);
				break;
			case "admin.user.delete":
				$this->setScreen("userlist");		
				IAUser::delete($id);
				break;
			case "admin.user.edit":
				$this->setScreen("useredit");			
				$this->refIAEditUser = new IAUser($this->id,$this->db,$id);
				break;
			case "admin.user.add":
				$this->setScreen("useredit");			
				$this->refIAEditUser = new IAUser($this->id,$this->db);
				break;

			/* library admin */
			case "admin.library.update":
				$this->setScreen("collectionlist");
				$this->userUpdate($form);
				break;

			/* collection admin */
			case "admin.collection.list":
				$this->setScreen("collectionlist");
				break;
			case "admin.collection.moveup":
				$this->moveCollecion($id,-1);
				$this->setScreen("collectionlist");
				break;
			case "admin.collection.movedown":
				$this->moveCollecion($id,1);
				$this->setScreen("collectionlist");
				break;
			case "admin.collection.newfolderoptions":
				$this->setScreen("newfolderoptions");
				$this->refIACollection = new IACollection($this,$this->db);
				break;
			case "admin.collection.insert":
				$this->setScreen("collectionedit");			
				$this->refIACollection->insert($form);
				break;
			case "admin.collection.importoptions":
				$this->setScreen("importoptions");
				$this->refIACollection = new IACollection($this,$this->db);
				$this->refIACollection->setFolder($id);
				break;
			case "admin.collection.import":
				$this->setScreen("collectionedit");			
				$this->refIACollection->importToDB($id,$form);
				break;
			case "admin.collection.importfolder":
				$this->setScreen("collectionlist");			
				$col = new IACollection($this,$this->db);
				$col->importToDB($id);
				break;
			case "admin.collection.delete":
				$this->setScreen("collectionlist");			
				$col = new IACollection($this,$this->db,$id);
				$col->deleteFromDB();
				break;
			case "admin.collection.userlist":
				$this->setScreen("collectionusers");
				$this->setCollection($id,$sub,$sval,"user");
				break;
			case "admin.collection.update":
				$this->setScreen("collectionedit");
				$this->updateCollecion($id,$form);
				break;			
			case "admin.collection.edit":
				$this->setScreen("collectionedit");
				$this->setCollection($id,$sub,$sval,"user");
				break;
			case "admin.collection.imageupload":
				$this->setScreen("collectionedit"); //temporary
				if ($this->refIACollection->importImage($form) != true)
					$this->setAlert($GLOBALS["strUploadError"]);
				break;
			case "admin.collection.imagedelete":
				$this->setScreen("collectionedit");
				$this->refIACollection->deleteImage($id);
				break;
			case "admin.collection.imageedit":
				$this->setScreen("imageedit");
				$this->setImage($id,$sub);
				break;
			case "admin.collection.imagelistsave":
				$this->setScreen("collectionedit");
				$this->saveImageList($id,$form);
				break;
			case "admin.collection.imagesave":
				$this->saveImage($id,$form);
				break;
			case "admin.collection.deletecomment":
				$this->setScreen("imageedit");
				$this->deleteComment($id,$form);
				break;
			/* collection group */
			case "collection.view":
				$this->setScreen("collectionview");
				$clist = $this->refIAUser->collectionsRel("id");
				if (in_array($id,$clist)) {
					$this->setCollection($id,$sub,$sval,"group");
				} else {
					$this->setCollection($id,$sub,$sval,"others");
				}
				break;
			case "collection.imageview":
				$this->setScreen("imageview");
				$this->setImage($id,$sub,true);
				break;
			case "collection.addcomment":
				$this->setScreen("imageview");
				$this->addComment($form);
				break;
			case "collection.showcomments":
				$this->setScreen("imageview");
				if($this->refIACollection)
					$this->refIACollection->commentsSetVisible(true);
				break;
			case "collection.hidecomments":
				$this->setScreen("imageview");
				if($this->refIACollection)
					$this->refIACollection->commentsSetVisible(false);
				break;
			
			/* other actions */
			case "authenticate":
				$this->refIAUser->authenticate($this->id,$form[username],$form[password]);
				break;
			case "logout":
				$this->refIAUser->init();
				$this->refIACollection = NULL;
				if ($this->getField("guestonly") == "yes")
					$this->forceLogin = false;
				break;
			case "notpermitted":
				$this->setScreen("notpermitted");
				break;
			case "showlogin":
				$this->forceLogin = true;
				break;
			default:
				break;
		}
	}

	/**
	 *
	 */
	function getAlertPane() {
		$out = "";
		if (is_array($this->alert)) {
			$tpl = &$this->getTemplateInstance("alert",true);
			$tpl->setVariable("Title", $this->alert[0]);
			$tpl->setVariable("Text", $this->alert[1]);
			$out = $tpl->get();
			$this->alert = null;
		}
		return $out;
	}

	
	/**
	 *
	 */
	function getDataFromDB() {
		$sql = "SELECT * FROM `iadomain` WHERE `id` = $this->id";
		$res = $this->db->query($sql);
		
		if($row = $res->fetchRow(DB_FETCHMODE_ASSOC))
			$this->fields = $row;
		else
			$this->id = NULL;
	}

	/**
	 * 
	 */
	function authenticate($name) {

		$sql = "SELECT `id` FROM `iadomain` WHERE `name` = '$name' LIMIT 1";
		$res = $this->db->query($sql);
						
		if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$this->id = $row['id'];
			$this->getDataFromDB();
			return true;
		} else {
			return false;
		}
	}

	/**
	 *
	 */
	function getDataPath($root) {
		$path = $root;
	
		if ($this->id) {
			$folder = $this->getField("folder");
			if (strlen($folder))
				$path .= "/".$folder;
		}
		
		return $path;
	}

	/**
	 *
	 */
	function getFields() {
		return $this->fields;
	}

	/**
	 *
	 */
	function getField($key) {
		return $this->fields[$key];
	}

	/**
	 *
	 */
	function &getTemplateInstance($name,$removeEmptyBlocks=true,$formaction=null) {
		$tplName = $this->getField("htmlTemplate");
		$tplFile = $tplName."/$name.template.html";
				
		if (!file_exists("./appearance/".$tplFile))
			$tplFile = "default/$name.template.html";

		$tpl = new HTML_Template_IT("./appearance");
		$tpl->loadTemplatefile($tplFile, true, $removeEmptyBlocks);

		if ($formaction) {
			$tpl->setVariable("PHP_SELF",$_SERVER['PHP_SELF']);
			$tpl->setVariable("ACTION",$formaction);
		}
		
		return $tpl;
	}

	/**
	 * 
	 */
	function domainView() {
		$noFolders = ($this->cfg['datafolders'] == false);
		$yesNo = array("Yes"=>"yes","No"=>"no");
		$templateList = $this->templateList("main");

		$tpl = &$this->getTemplateInstance("admin.domain",true,"domain.update");
		$tpl->setVariable("DomainListLink", IAHTML::linkLocation("domain.list"));
		$tpl->setVariable("DOMAINNAME", $this->getField("name"));
		$tpl->setVariable("ID", $this->id);
		$tpl->setVariable("name", $this->getField("name"));
		
		if ($noFolders) {
			$tpl->setVariable("httpUpload",IAHTML::getFixValue("httpUpload","yes"));
			$tpl->setVariable("folder", "[no folder]");
			$tpl->setVariable("READONLY", "readonly");
		} else {
			$tpl->setVariable("httpUpload",IAHTML::getFormPopup("httpUpload",$this->getField("httpUpload"),$yesNo));
			$tpl->setVariable("folder", $this->getField("folder"));
		}

		$tpl->setVariable("guestonly",IAHTML::getFormPopup("guestonly",$this->getField("guestonly"),$yesNo));
		$tpl->setVariable("htmlTemplate",IAHTML::getFormPopup("htmlTemplate",$this->getField("htmlTemplate"),$templateList));

		$tpl->setVariable("defaultdomain",IAHTML::getFormCheckbox("defaultdomain","yes",$this->getField("defaultdomain")=="yes"));

		$out = $tpl->get();
		return $out;
	}


	/**
	 * 
	 */
	function navigation() {
		$tpl = &$this->getTemplateInstance("navigation",true);
		$tpl->setVariable("HomeLink", $_SERVER['PHP_SELF']);

		if ($this->getField("guestonly") == "yes" && $this->forceLogin == false) {
   
        	$tpl->setCurrentBlock("hiddenlogin");
			$tpl->setVariable("ShowLoginLink", $_SERVER['PHP_SELF']."?action=showlogin") ;
    	    $tpl->parseCurrentBlock("hiddenlogin") ;

		} elseif ($this->refIAUser->isAuthenticated()) {

        	$tpl->setCurrentBlock("userinfo");
			$tpl->setVariable("UserFullname", $this->refIAUser->fullname) ;
			$tpl->setVariable("LogoutLink", $_SERVER['PHP_SELF']."?action=logout") ;
    	    $tpl->parseCurrentBlock("cell") ;
        			
			$clist = $this->refIAUser->collectionsRel();
			
			if (count($clist) > 0) {
		
				foreach ($clist as $row) {
					$tpl->setCurrentBlock("user_collection");
					$tpl->setVariable("UserCollectionLink", $_SERVER['PHP_SELF']."?action=collection.view&amp;id=".$row["id"]);
					$tpl->setVariable("UserCollectionName", $row["name"]);
					$tpl->parseCurrentBlock("user_collection");
				}
	
				$tpl->setCurrentBlock("user_collection_list");
				$tpl->setVariable("UserCollections",$GLOBALS['strUserCollections']);
				$tpl->parseCurrentBlock("user_collection_list") ;
			}
		} else {
			$tpl->setCurrentBlock("login");
			$tpl->setVariable("PHP_SELF", $_SERVER['PHP_SELF']);
			$tpl->setVariable("USERNAME", $GLOBALS['strUsername']);
			$tpl->setVariable("PASSWORD", $GLOBALS['strPassword']);
			$tpl->setVariable("ACTIONFIELD", "<input type=\"hidden\" name=\"action\" value=\"authenticate\">");
			$tpl->parseCurrentBlock("login") ;
		}
		
		$pclist = $this->publicList();

		if (count($pclist) > 0) {
			
			foreach ($pclist as $row) {
				$tpl->setCurrentBlock("public_collection");
				$tpl->setVariable("PublicCollectionLink", $_SERVER['PHP_SELF']."?action=collection.view&amp;id=".$row["id"]);
				$tpl->setVariable("PublicCollectionName", $row["name"]);
				$tpl->parseCurrentBlock("public_collection");
			}

			$tpl->setCurrentBlock("public_collection_list");
			$tpl->setVariable("PublicCollections",$GLOBALS['strPublicCollections']);
			$tpl->parseCurrentBlock("public_collection_list") ;
		}

		if ($this->refIAUser->isAdmin()) {
			$tpl->setCurrentBlock("admin");
			$tpl->setVariable("AdminCollectionsLink", $_SERVER['PHP_SELF']."?action=admin.collection.list");
			$tpl->setVariable("AdminUsersLink", $_SERVER['PHP_SELF']."?action=admin.user.list");
			$tpl->parseCurrentBlock("admin") ;
		}

	//	$table->addBreak();
	//	$table->addLine(array("<img src=\"images/home.gif\" width=\"16\" height=\"16\">"),array(""=>"Home"));
		
		return $tpl->get();
	}


	/**
	 * 
	 */
	function content() {

		switch($this->screen) {
			case "userlist":
				$out = $this->userList();
				break;
			case "useredit":
				$out = $this->refIAEditUser->userView();
				break;
			case "importoptions":
				$out = $this->refIACollection->importOptions();
				break;
			case "newfolderoptions":
				$out = $this->refIACollection->newFolderOptions();
				break;
			case "collectionlist":
				$out = $this->collectionList();
				break;
			case "collectionedit":
				$out = $this->refIACollection->imageList(true);
				break;
			case "collectionusers":
				$out = $this->refIACollection->userList();
				break;
			case "collectionview":
				$out = $this->refIACollection->imageList();
				break;
			case "imageedit":
				$out = $this->refIACollection->imageView(true);
				break;
			case "imageview":
				$out = $this->refIACollection->imageView();
				break;
			default:
				$out = $this->getIndex();
				break;
		}
		return $this->getAlertPane().$out;
	}

	/**
	 *
	 */
	function imageIsAllowed($img) {
		return (in_array($img,$this->allowedImages));
	}	
	
	/**
	 *
	 */
	function getIndex() {
		$tpl = &$this->getTemplateInstance("splash",true);
		$tpl->setCurrentBlock("title");
		$tpl->setVariable("title",$this->getField("title"));
		$tpl->setVariable("description",$this->getField("description"));
		$tpl->parseCurrentBlock("title");

		$list = $this->publicList();
		if ($this->refIAUser->isAuthenticated()) {
			$clist = $this->refIAUser->collectionsRel();
			$list = array_merge($list,$clist);
		}
		
		$this->allowedImages = array();
		
		foreach ($list as $col) {
			$previewtype = $col['preview'];
			$img_src = "";
			
        	if (substr_count($previewtype,"image") > 0) {
        		$sql = "SELECT path FROM `iaimage` WHERE `collection_id` = {$col['id']} AND access != 'user' AND access != 'group' ORDER BY visits DESC LIMIT 1";
        		$res = $this->db->query($sql);
        		if ($data = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
	        		$this->allowedImages[] = $data['path'];
        			$img_src = "stream.php?view=thumbnail&amp;path=".urlencode($data['path']);
        		}
        	}

        	$tpl->setCurrentBlock($previewtype);
		
        	switch ($previewtype) {
        		case "image_and_text":
					$tpl->setVariable("it_link", $_SERVER['PHP_SELF']."?action=collection.view&amp;id=".$col["id"]);
					$tpl->setVariable("it_name", $col['name']);
					$tpl->setVariable("it_description", $col['description']);
					$tpl->setVariable("it_src", $img_src);
					break;
				case "image":
					$tpl->setVariable("i_link", $_SERVER['PHP_SELF']."?action=collection.view&amp;id=".$col["id"]);
					$tpl->setVariable("i_name", $col['name']);
					$tpl->setVariable("i_src", $img_src);
					break; 
				case "text":
					$tpl->setVariable("t_link", $_SERVER['PHP_SELF']."?action=collection.view&amp;id=".$col["id"]);
 					$tpl->setVariable("t_name", $col['name']);
					$tpl->setVariable("t_description", $col['description']);
					break;
        	}
    	    $tpl->parseCurrentBlock($previewtype) ;
		}

		return $tpl->get();
	}

	/**
	 * 
	 */
	function setCollection($id,$sub,$sval,$mode) {
	
		$this->collectionAccessLevel($id);
	
		if($id && ($this->refIACollection == NULL || $this->refIACollection->getID() != $id))
			$this->refIACollection = new IACollection($this,$this->db,$id,$mode);
			
		if($id && $this->refIACollection) {
			$this->refIACollection->setMode($mode);
			$this->refIACollection->getDataFromDB();
		}
					
		if($this->refIACollection)
			$this->refIACollection->flow($sub,$sval);
	}
	
	
	/**
	 * checks, if the current user is admitted to a collection
	 *
	 * @param collectionID	collection id
	 */
	function collectionAccessLevel($collectionID) {
		$ret = false;
		if ($this->refIAUser->isAuthenticated()) {
			$clist = $this->refIAUser->collectionsRel("id");
			$plist = $this->publicList("id");
			if (in_array($collectionID,$clist))
				$ret = "group";
			elseif (in_array($collectionID,$plist))
				$ret = "others";
		} else {
			$list = $this->publicList("id");
			if (in_array($collectionID,$list))
				$ret = "others";
		}
		return $ret;		
	}

	/**
	 * 
	 */
	function moveCollecion($id,$delta) {
		$sql = "SELECT id,position FROM `iacollection` WHERE `domain_id` = $this->id AND id = $id LIMIT 1";
		$res = $this->db->query($sql);
			
		if ($data = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$oldposition = $data['position'];
			$newposition = $oldposition + $delta;
		
			if ($newposition < 1)
				return;
				
			$sql = "SELECT `id`,`position` FROM `iacollection` WHERE `domain_id` = $this->id AND `position` = $newposition LIMIT 1";
			$res2 = $this->db->query($sql);

			if ($data2 = $res2->fetchRow(DB_FETCHMODE_ASSOC)) {
				$otherid = $data2['id'];
				
				$sql = "UPDATE `iacollection` SET `position` = $newposition WHERE `id` = $id";
				$trs = $this->db->query($sql);		

				$sql = "UPDATE `iacollection` SET `position` = $oldposition WHERE `id` = $otherid";
				$trs = $this->db->query($sql);
			}
		}
	}
	
	/**
	 * 
	 */
	function updateCollecion($id,$form) {
		if($this->refIACollection) {
			$this->refIACollection->update($form);
		}	
	}
	
	/**
	 * 
	 */
	function updateUser($id,$form) {
		if($this->refIAEditUser) {
			$this->refIAEditUser->update($form);
		}	
	}	
	
	/**
	 *
	 */
	function setImage($id,$sub,$view=false) {
		
		$imageOK = false;
		$imageAccess = "";
		$collectionID = 0;
		$collectionOK = false;
		$collectionAccess = "";
		
		if (!$id) {
			$this->setScreen("notpermitted");
			return;
		}
		
		if ($view) {
			$sql = "SELECT * FROM `iaimage` WHERE `id` = $id AND `domain_id` = $this->id LIMIT 1";
			$res = $this->db->query($sql);
			
			if ($data = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$collectionID = $data["collection_id"];
				$imageAccess = $data["access"];
				$imageOK = true;
			}
	
			$sql = "SELECT * FROM `iacollection` WHERE `id` = $collectionID AND `domain_id` = $this->id LIMIT 1";
			$res = $this->db->query($sql);
			
			if ($data = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$collectionAccess = $data["access"];
				$collectionOK = true;
			}
			
			if($this->refIACollection)
				$matchingCollection = ($this->refIACollection->getID() == $collectionID);
			
			$accessLevel = $this->collectionAccessLevel($collectionID);
			
			if ($imageOK && $collectionOK && $accessLevel) {
			
				if (!$matchingCollection)
					$this->setCollection($collectionID,$sub,$sval,$access);

				if (($accessLevel == "group" && $imageAccess != "user") || ($accessLevel == "others" && $imageAccess == "others")) {
					$this->refIACollection->flow("setimage",$id);
				} else {
					$this->setScreen("notpermitted");
				}
			} else {
				$this->setScreen("notpermitted");
			}
		} elseif ($this->refIACollection) {
			$this->refIACollection->flow("setimage",$id);
		} else {
			$this->setScreen("notpermitted");
		}
	}
		
	/**
	 * 
	 */
	function addComment($form) {
		if($this->refIACollection)
			$this->refIACollection->addComment($form);
	}
	
	/**
	 * 
	 */
	function saveImage($id,$form) {
		if($id && ($this->refIACollection)) {
			$prev = NULL;
			$next = NULL;
		
			$this->refIACollection->imageSave($id,$form);
			$this->refIACollection->imageNeighbors($id,$prev,$next);
			
			if (isset($form["prev"]) && $prev) {
				$this->setImage($prev,$sub);
				$this->setScreen("imageedit");
			} elseif (isset($form["next"]) && $next) {
				$this->setImage($next,$sub);
				$this->setScreen("imageedit");
			} else {
				$this->setScreen("collectionedit");
			}
		}
	}

	/**
	 *
	 */
	function deleteComment($id,$form) {	
		if($id && ($this->refIACollection)) {
			$this->refIACollection->deleteComments($id,$form);
		}
	}

	/**
	 * 
	 */
	function saveImageList($id,$form) {
		if(is_array($id) && ($this->refIACollection)) {
			$this->refIACollection->imageListSave($id,$form);
			
			if (isset($form["prev"]))
				$this->refIACollection->flow("prevpage",null);
			elseif (isset($form["next"]))
				$this->refIACollection->flow("nextpage",null);
		}
	}
	
	/**
	 * 
	 */
	function dataEdit() {
		$tpl = &$this->getTemplateInstance("admin.gallery",false);
		$tpl->setVariable("PHP_SELF", $_SERVER['PHP_SELF']);
		$tpl->setVariable("ACTIONFIELD", "<input type=\"hidden\" name=\"action\" value=\"admin.library.update\">");
		$tpl->setVariable("title",$this->getField("title"));
		$tpl->setVariable("description",$this->getField("description"));
		$tpl->setVariable("htmlTemplate",IAHTML::getFormPopup("htmlTemplate",$this->getField("htmlTemplate"),$this->templateList()));
		$tpl->setVariable("guestonly",IAHTML::getYesNoPopup("guestonly",$this->getField("guestonly")));

		return $tpl->get();
	}
	

	/**
	 * 
	 */
	function collectionList() {
		$out = "\n<div class=\"navi01\">Admin</div>";

		$out .= $this->dataEdit();

		$collFolders = $this->folderList();
		$dbFolders = array();

		$sql = "SELECT id,position,name,folder,access,preview FROM `iacollection` WHERE `domain_id` = $this->id ORDER BY position, name";
		
		$res = $this->db->query($sql);
	
		$tpl = &$this->getTemplateInstance("admin.collectionlist");
		$cpos = 1;

		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if ($row['position'] != $cpos) {
				$sql = "UPDATE `iacollection` SET `position` = $cpos WHERE id=".$row['id'];
				$trs = $this->db->query($sql);
			}
			unset($row['position']);
			$cpos++;
			
			$tpl->setCurrentBlock("collection");
			
			// values
			$tpl->setVariable("NAME",$row['name']);
			$tpl->setVariable("FOLDER",$row['folder']);
			$tpl->setVariable("ACCESS",$row['access']);
			$tpl->setVariable("PREVIEW",$row['preview']);
			
			// actions
			$tpl->setVariable("MoveUpLink",IAHTML::linkLocation("admin.collection.moveup", array("id"=>$row['id'])));
			$tpl->setVariable("MoveDownLink",IAHTML::linkLocation("admin.collection.movedown", array("id"=>$row['id'])));
			$tpl->setVariable("EditLink",IAHTML::linkLocation("admin.collection.edit", array("id"=>$row['id'])));
			$tpl->setVariable("DeleteLink",IAHTML::deleteLinkLocation("admin.collection.delete", array("id"=>$row['id'])));

			$tpl->parseCurrentBlock("collection");

			$dbFolders[] = $row["folder"];
		}

		if ($this->getField("httpUpload") == "yes") {
			$newLabel = ($this->cfg['datafolders']) ? "New Folder" : "Add Collection";
			$tpl->setCurrentBlock("newcollection");
			$tpl->setVariable("NewCollectionButton",$newLabel);
			$tpl->setVariable("ACTION","admin.collection.newfolderoptions");
			$tpl->setVariable("PHP_SELF",$GLOBALS['PHP_SELF']);
			$tpl->parseCurrentBlock("newcollection");
		} else {
			$tpl->setVariable("STARTHIDE","<!--");
			$tpl->setVariable("ENDHIDE","-->");

			foreach ($collFolders as $folder) {
				if (!in_array($folder,$dbFolders)) {
					$tpl->setCurrentBlock("folder");
					$tpl->setVariable("NEWFOLDER",$folder);
					$tpl->setVariable("ImportOptionsLink",IAHTML::linkLocation("admin.collection.importoptions", array("id"=>$folder)));
					$tpl->parseCurrentBlock("collection");
				}
			}
		}
		
		return $out.$tpl->get();
	}
	
	/**
	 * 
	 */
	function userList() {
		$out = "\n<div class=\"navi01\">Admin</div>";

		$sql = "SELECT id,name,username,password,rights FROM `iauser` WHERE `domain_id` = $this->id ORDER BY name";
		$res = $this->db->query($sql);
		
		$table = new IATable("list");
		$table->setNumCellsInRow(4);
		$table->addHeaderLine("<img src=\"images/users.gif\" width=\"16\" height=\"16\" align=\"right\">User List","sub1");
		$table->addHeaderLine(array("Name","Login","Rights","Action"),"sub2");
					
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$table->addLine($row,array("admin.user.edit"=>"<img border=\"0\" src=\"images/edit.gif\" alt=\"Edit\" width=\"16\" height=\"16\"> Edit","admin.user.delete"=>"<img border=\"0\" src=\"images/destroy.gif\" alt=\"Destroy\" width=\"16\" height=\"16\"> Delete"),"id");
		}
	
		$out .= $table->toHTML(2,1);
		
		$out .= IAHTML::formStart(array("id"=>-1,"action"=>"admin.user.add"));
		$out .= "<p>&nbsp;&nbsp;<input type=\"submit\" value=\"Add User\" class=\"varbutton\"></p>";
		$out .= IAHTML::formEnd();

		return $out;
	}


	/**
	 * 
	 */
	function publicList($field=NULL) {
		$ret = array();
		
		if ($this->refIAUser->isAuthenticated())
			$colRel = $this->refIAUser->collectionsRel("id");
		else
			$colRel = array();
		
		$sql = "SELECT * FROM `iacollection` WHERE `access` = 'others' AND `domain_id` = $this->id ORDER BY position";
		$res = $this->db->query($sql);

		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if (!in_array($row["id"],$colRel))
				$ret[] = ($field) ? $row[$field] : $row;
		}
		
		return $ret;
	}


	/**
	 * 
	 */
	function delete() {
		if ($this->id == NULL)
			return;
			
		$sql = "SELECT id FROM `iacollection` WHERE `domain_id` = $this->id";
		$res = $this->db->query($sql);
		$cids = array();
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$cids[] = $row['id'];
		}
		
		foreach ($cids as $cid) {
			$col = new IACollection($this,$this->db,$cid);
			$col->deleteFromDB();
		}
		
		$sql = "DELETE FROM `iadomain` WHERE id=$this->id";
		$res = $this->db->query($sql);

		$sql = "DELETE FROM `iauser` WHERE `domain_id`=$this->id";
		$res = $this->db->query($sql);
	}

	/**
	 * 
	 */
	function userUpdate($form) {
		assert($this->id != null);
		$sql = "UPDATE IADomain SET title='$form[title]',description='$form[description]',htmlTemplate='$form[htmlTemplate]',guestonly='$form[guestonly]' WHERE `id` = $this->id";
		$res = $this->db->query($sql);
		$this->getDataFromDB();
	}

	/**
	 * 
	 */
	function update($form) {

		if ($form["defaultdomain"] == "yes") {
			$sql = "UPDATE IADomain SET defaultdomain='no'";
			$res = $this->db->query($sql);				
		}
	
		$checkedDomainName = $this->checkFolderName($form["name"]);
		$checkedFolderName = $this->checkFolderName($form["folder"]);

		if ($checkedDomainName != $form["name"] || $checkedFolderName != $form["folder"])
			$ret = 0;
		elseif ($this->id) {
			$oldpath = $this->rootPath;
			$newpath = substr($this->rootPath, 0, strlen($this->rootPath) - strlen($this->getField("folder"))).$form["folder"];
			$aOK = rename($oldpath.".thumbnail", $newpath.".thumbnail");
			$aOK = rename($oldpath, $newpath);
			$setDefaultDomain = ($form["defaultdomain"] == "yes") ? ",defaultdomain='yes'" : ",defaultdomain='no'";
		
			//echo $form["defaultdomain"],$setDefaultDomain;
		
			if ($aOK) {
				$sql = "UPDATE IADomain SET name='$form[name]',folder='$form[folder]',htmlTemplate='$form[htmlTemplate]',httpUpload='$form[httpUpload]',guestonly='$form[guestonly]'$setDefaultDomain WHERE `id` = $this->id";
				$res = $this->db->query($sql);

				$this->getDataFromDB();
				$ret = 1;
			} else {
				$ret = 0;
			}
		} else {
			if (!isset($form['folder']))	// when $cfg['datafolders'] == false
				$form['folder'] = "";
		
			$sql = "INSERT INTO IADomain (name,folder,htmlTemplate,httpUpload,guestonly,defaultdomain) VALUES ('$form[name]','$form[folder]','$form[htmlTemplate]','$form[httpUpload]','$form[guestonly]','$form[defaultdomain]')";
			$res = $this->db->query($sql);
			
			$sql = "SELECT LAST_INSERT_ID()";
			$res = $this->db->query($sql);
			$row = $res->fetchRow();
		
			$this->id = $row[0];

			$sql = "INSERT INTO `iauser` (domain_id,name,rights,username,password) VALUES ($this->id,'Administrator','admin','admin','admin')";
			$res = $this->db->query($sql);
			$ret = 2;
			
			// Add folders when using http upload
			if ($form['httpUpload'] == "yes" && strlen($form['folder']) > 0 ) {
				$path = $this->rootPath."/".$form['folder'];
				$c1 = file_exists($this->rootPath);
				$c2 = file_exists($path);
				$c3 = file_exists($path.".thumbnail");
				
				if ($c1 && !$c2) {
					umask(0000);
					mkdir($path,0777);
					$ret = 3;
				}

				if ($c1 && !$c3) {
					umask(0000);
					mkdir($path.".thumbnail",0777);
				}
			}
		}
		return $ret;
	}


	/**
	 * strips all non suitable characters for domain names from a string
	 */
	function checkFolderName($name) {
		$length = strlen($name);
		$ret = "";

		if ($length == 0 || $name == "." || $name == "..")
			return null;
		
		for ($i = 0; $i < $length; $i++) {
			$c = $name{$i};
			if (ereg("[a-zA-Z0-9.-_]",$c))
				$ret .= $c;
		}
		
		return $ret;
	}


	/**
	 * 
	 */
	function templateList($prefix="") {
		$appearanceDir = "appearance";
		$list = array();
		$d = dir($appearanceDir);
		
		while($entry=$d->read()) {
			if (is_dir($appearanceDir."/".$entry) && $entry[0] != "." && $entry != "CVS")
				$list[$entry] = $entry;
		}
		
		$d->close();
		
		return $list;
	}

	/**
	 * 
	 */
	function folderList() {

		$folderList = array();
		$d = dir($this->rootPath);
		
		while($entry=$d->read()) {
			if (is_dir($this->rootPath."/$entry") && $entry[0] != '.' && $entry != "thumbnail")
				$folderList[] = $entry;
		}
		
		$d->close();
		
		return $folderList;
	}
}

?>
