<?php

// +----------------------------------------------------------------------+
// | ImageAlbum version 2.0                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002-2004 Joachim Fornallaz                            |
// +----------------------------------------------------------------------+
// | This source file is subject to the GPL license.                      |
// +----------------------------------------------------------------------+
// | Authors: Joachim Fornallaz <jf@omnis.ch>                             |
// +----------------------------------------------------------------------+

/* $Id: IATest.php,v 1.4 2004/03/10 01:02:03 jfornall Exp $ */

require_once("DB.php");

class IATest {

	function displayVariable($names) {
		global $cfg;
		
		if (!is_array($names))
			$names = array($names);
	
		echo "\n<div class=\"code\">";
		foreach ($names as $name) {
			$value = isset($cfg[$name]) ? $cfg[$name] : $GLOBALS[$name];
			$vname = isset($cfg[$name]) ? "\$cfg['$name']" : "\$$name";
			
			if (is_bool($value))
				$valstr = ($value) ? "true" : "false";
			else
				$valstr = "\"$value\"";
			
			echo "$vname = $valstr;<br />";
		}
		echo "</div>";
	}
	
	function testDisplay($succeeded, $textOK, $textError, $varnames = null) {
		$img = $succeeded ? "checkOK.png" : "checkNO.png";
		$text = $succeeded ? $textOK : $textError;
		echo "\n<p><img src=\"images/$img\" alt=\"\" width=\"14\" height=\"14\" align=\"left\" />&nbsp;$text";
	
		if (!$succeeded && $varnames != null) {
			echo " You might want to check following lines in the configuration file:";
			displayVariable($varnames);
		}
		echo "</p>";
	}
	
	function testPHP() {
		$thisPHPversion = phpversion();
		$isPHP430 = version_compare($thisPHPversion, "4.3.0", ">=");
		
		$succeeded = $isPHP430;
		IATest::testDisplay($succeeded,"PHP version $thisPHPversion", "Not supported PHP version ($thisPHPversion). ImageAlbum requires version 4.3.0 or higher.");
	}
	
	function testImageFunctions() {
		$succeeded = (function_exists("imagecopyresampled") == true);
		$succeedMsg = "Image manipulation functions available, using GD library.";
	
		if ($succeeded == false) {
			$out = array();
			$cmd = $GLOBALS["binarypath"]."/montage";
			$ret = exec($cmd, $out);
			
			if ($ret == true) {
				$imInfoArray = explode(" ",$out[0]);
				$imInfo = $imInfoArray[1]." ".$imInfoArray[2];
				
				$succeeded = true;
				$succeedMsg = "Image manipulation functions available, using  $imInfo";
			}
		}
	
		$safemode = ini_get("safe_mode");
		$failedMsg = "No image manipulation functions available, ImageAlbum requires PHP with GD extension (version 2.0 or newer) enabled.";
	
		IATest::testDisplay($succeeded,$succeedMsg,$failedMsg);
	}
	
	// Database
	
	function testDatabaseType($cfg,$varname) {
		$type = $cfg[$varname];
		$dbTypes = array(
			"mysql"  => "MySQL",
			"pgsql"  => "PostgreSQL",
			"ibase"  => "InterBase",
			"msql"   => "Mini SQL",
			"mssql"  => "Microsoft SQL Server",
			"oci8"   => "Oracle 7/8/8i",
			"odbc"   => "ODBC (Open Database Connectivity)",
			"sybase" => "SyBase",
			"ifx"    => "Informix",
			"fbsql"  => "FrontBase");
		
		$succeeded = $dbTypes[$type] != null;
		IATest::testDisplay($succeeded,"Type: ".$dbTypes[$type],"Not supported type ($type).",$varname);
		
		return $succeeded;
	}
	
	function getDSN($cfg) {
		$dbType = $cfg['dbType'];
		$dbHost = $cfg['dbHost'];
		$dbName = $cfg['dbName'];
		$dbUsername = $cfg['dbUsername'];
		$dbPassword = $cfg['dbPassword'];
	
		$dsn = "$dbType://$dbUsername:$dbPassword@$dbHost/$dbName";
		return $dsn;
	}
	
	function testDatabaseConnection($cfg,$varnames) {
		$dsn = IATest::getDSN($cfg);
		$db = DB::connect($dsn, true);
		$dbError = DB::isError($db);
		$errMsg = $dbError ? $db->getMessage() : "";
		
		IATest::testDisplay(!$dbError,"Connection OK", $errMsg,$varnames);
		
		if (!$dbError)
			$db->disconnect();
			
		return !$dbError;
	}
	
	function testDatabaseTables($cfg) {
		$dsn = IATest::getDSN($cfg);
		$db = DB::connect($dsn, true);
		$dbError = DB::isError($db);
	
		$failureMsg = "Could not check database tables.";
		
		if ($cfg['dbType'] == "mysql") {
			$sql = "SHOW TABLES";
			$res = $db->query($sql);
			$iaTables = array("iacollection","iacomment","iadomain","iaimage","iauser","iausercollection");		
	
			while ($row = $res->fetchRow()) {
				$table = $row[0];
				$key = array_search($table,$iaTables);
	
				if (is_int($key)) {
					unset($iaTables[$key]);
				}
			}
			
			if (count($iaTables) > 0) {
				$failureMsg = "Tables missing:";
				
				foreach ($iaTables as $oneTable) {
					$failureMsg .= " $oneTable";
				}
			}
			$succeeded = (count($iaTables) == 0);
			$succeedMsg = "All required tables exist.";
			$failureMsg .= ". The sql script for the ImageAlbum tables can be found in the 'scripts' folder.";
		}
		IATest::testDisplay($succeeded,$succeedMsg,$failureMsg);
	
		if (!$dbError)
			$db->disconnect();
		
		return $succeeded;
	}
	
	function testDefaultDomain($cfg) {
		$dsn = IATest::getDSN($cfg);
		$db = DB::connect($dsn, true);
		$dbError = DB::isError($db);
	
		if (!$dbError)
			$db->disconnect();
	}
	
	// Application Settings 
	
	function testLanguage($cfg,$varname) {
		$langcode = $cfg[$varname];
	
		$languages = array(
			"en"  => "English",
			"de"  => "German");
	
		$succeeded = $languages[$langcode] != null;
		IATest::testDisplay($succeeded,"Language: ".$languages[$langcode],"Not supported language ($langcode).",$varname);
	}
	
	function testDatapath($cfg,$varname) {
		$path = $cfg[$varname];
		
		clearstatcache(); 
		$dir = $path;
		$pathOK = is_dir($dir);
		IATest::testDisplay($pathOK,"Datapath exists.", "Datapath does not (yet) exist.",$varname);
		
		if ($pathOK) {
			$writeOK = is_writeable($dir);
			IATest::testDisplay($writeOK,"Datapath is writeable.", "Datapath '$path' is not writeable. Change the write permissions of this folder.");
		}
		
		return $writeOK && $pathOK;
	}
	
	function testDatafolders($cfg,$varname) {
		$datafolders = $cfg[$varname];
		
		$safemode = ini_get("safe_mode"); 
		$succeeded = ($datafolders == false || $safemode == false);
		$safeON = "PHP safe mode is " . ($safemode ? "on" : "off");
		$textOK = $datafolders ? "Using real folder structures for data storage is OK." : "Storing all data in one directory.";
	
		IATest::testDisplay($succeeded,"$safeON. $textOK", "$safeON. Please set folowing setting to <tt>false</tt>.",$varname);
		
		return $succeeded;
	}
	
	function testAdminDomain($cfg,$varnames) {
		$admindomain = $cfg['admindomain'];
		$adminlogin = $cfg['adminlogin'];
		$adminpasswd = $cfg['adminpasswd'];
	
		$succeeded = ($admindomain != "admin" || $adminlogin != "admin" || $adminpasswd != "admin");
		
		$appRootURL = dirname($_SERVER['SCRIPT_NAME']);
		$scriptURL = "http://".$_SERVER['SERVER_NAME'].$appRootURL."/index.php/$admindomain/";
		$adminDomainLink = "Admin Domain: <a href='$scriptURL'>$scriptURL</a>";
		IATest::testDisplay($succeeded,"$adminDomainLink","$adminDomainLink. Please change the access codes.",$varname);
	}
}

?>