<?php

// +----------------------------------------------------------------------+
// | ImageAlbum version 2.0                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002-2004 Joachim Fornallaz                            |
// +----------------------------------------------------------------------+
// | This source file is subject to the GPL license.                      |
// +----------------------------------------------------------------------+
// | Authors: Joachim Fornallaz <jf@omnis.ch>                             |
// +----------------------------------------------------------------------+

/* $Id: ImageAlbum.php,v 1.11 2004/03/10 00:50:27 jfornall Exp $ */

require_once("DB.php");
require_once("classes/IADomain.php");
require_once("classes/IAUser.php");
require_once("classes/IACollection.php");
require_once("classes/IATable.php");

class ImageAlbum {

	// Variables

	var $appName = "ImageAlbum";
	var $appVersion = "2.0 beta";
	var $appCopyright = "2002-2004 <a href=\"http://jf.omnis.ch/\">Joachim Fornallaz</a>";
	var $appWelcome = "<p>&nbsp;</p><p align=\"center\">Welcome to ImageAlbum!</p><p>&nbsp;</p>";

	var $db;
	var $dsn;
	var $screen;
	var $cfg;
	var $alert;
	
	var $refIADomain = NULL;
	var $refIAEditDomain = NULL;

	var $domain;
	var $adminDomainMode = false;
	var $adminDomainAccess = false;
	
	/**
	 * Contructor
	 *
	 * @param	$dsn
	 * @param	$cfg
	 */
	function ImageAlbum($dsn,$cfg) {
		$this->dsn = $dsn;
		$this->cfg = $cfg;
		$this->domain = "";
		$this->alert = NULL;
	}
	
	/**
	 * @param	$domain		domain to check
	 */
	function checkDomain($domain) {
		
		// admin domain handling
		if ($domain == $this->cfg["admindomain"]) {
			$this->adminDomainMode = true;
			$this->adminDomainAccess = false;
			$this->domain = $domain;
			
			$authOn = (strlen($this->cfg["adminpasswd"]) > 0);
			
			if (!$_SERVER['PHP_AUTH_USER'] && $authOn) {
				header("WWW-Authenticate: Basic realm=\"ImageAlbum Admin\"");
				header("HTTP/1.0 401 Unauthorized");
				echo "You must enter a valid login ID and password to access this resource.\n";
				echo $_SERVER['PHP_AUTH_USER'];
				exit;
			} elseif ($authOn) {
				if ($_SERVER['PHP_AUTH_USER'] != $this->cfg["adminlogin"] || $_SERVER['PHP_AUTH_PW'] != $this->cfg["adminpasswd"]) {
					header("WWW-Authenticate: Basic realm=\"ImageAlbum Admin\"");
					header("HTTP/1.0 401 Unauthorized");
					echo "You must enter a valid login ID and password to access this resource.\n";
					exit;
				} else {	// Domain Supervisor logged in successfully
					$this->adminDomainAccess = true;
				}
			}
		} elseif ($this->domain != $domain) {
			$this->adminDomainMode = false;
			$this->domain = $domain;
			$this->refIADomain = new IADomain($this->db,$this->cfg);
					
			if ($this->refIADomain->authenticate($domain)) {
				$this->refIADomain->setRootPath($this->cfg["datapath"]);
			} else {
				$this->refIADomain = NULL;
			}
		}
	}

	/**
	 * @param	$template	
	 */
	function getTemplate() {
		$ret = "";
	
		if ($this->refIADomain)
			$ret = $this->refIADomain->getField("htmlTemplate");
		else
			$ret = "default";
	
		return $ret;
	}
	
	/**
	 * @param	$thescreen	setter method
	 */
	function setScreen($thescreen) {
		$this->screen = $thescreen;
	}
	
	
	/**
	 * @param	$msg		message text for alert
	 */
	 function setAlert($msg) {
	 	$this->alert = $msg;
	 }
	
	
	/**
	 * application control function
	 *
	 * @param	$action		main action to perform
	 * @param	$id			value referring to main action
	 * @param	$sub		second action to perform
	 * @param	$id			value referring to second action
	 * @param	$form		array containing from data
	 */
	function flow($action,$id,$sub,$sval,$form) {

		$this->db = DB::connect($this->dsn, true);
		$this->setScreen("");

		if (DB::isError($this->db))
		    die ($this->db->getMessage());

		if (isset($_SERVER["PATH_INFO"])) {
			$domain = $_SERVER["PATH_INFO"];
			$domain = str_replace("/", "", $domain);
		} else {
			$domain = $_SERVER["HTTP_HOST"];
		}

		$this->checkDomain($domain);

		if ($this->adminDomainAccess && $this->adminDomainMode && (strlen($action) == 0)) {
			$action = "domain.list";
		}

		switch ($action) {
			/* domain admin */
			case "domain.list":
				$this->setScreen("domainlist");		
				break;
			case "domain.edit":
				$this->setScreen("domainedit");			
				$this->refIAEditDomain = new IADomain($this->db,$this->cfg,$id);
				break;
			case "domain.add":
				$this->setScreen("domainedit");			
				$this->refIAEditDomain = new IADomain($this->db,$this->cfg);
				break;
			case "domain.update":
				$this->setScreen("domainlist");
				$this->updateDomain($id,$form);
				break;
			case "domain.delete":
				$this->setScreen("domainlist");
				$this->refIAEditDomain = new IADomain($this->db,$this->cfg,$id);
				$this->refIAEditDomain->setRootPath($this->cfg["datapath"]);
				$this->refIAEditDomain->delete();
				break;
			case "notpermitted":
				$this->setScreen("notpermitted");
				break;
			default:
				if($this->refIADomain)
					$this->refIADomain->flow($action,$id,$sub,$sval,$form);
				break;
		}
	}
	
	/**
	 * 
	 */
	function domainList() {
		$sql = "SELECT id,name,folder,httpUpload,guestonly,defaultdomain FROM `iadomain`";
		$res = $this->db->query($sql);

		$tpl = &$this->getTemplateInstance("admin.domainlist",true,"domain.add");
		
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$tpl->setCurrentBlock("domain");
			if ($row['defaultdomain'] == "yes")
				$tpl->setVariable("HighlightClass","highlighted");

			// values
			$tpl->setVariable("name",$row['name']);
			$tpl->setVariable("folder",$row['folder']);
			$tpl->setVariable("httpUpload",$row['httpUpload']);
			$tpl->setVariable("guestonly",$row['guestonly']);

			$tpl->setVariable("EditLink",IAHTML::linkLocation("domain.edit", array("id"=>$row['id'])));
			$tpl->setVariable("DeleteLink",IAHTML::deleteLinkLocation("domain.delete", array("id"=>$row['id'])));

			$tpl->parseCurrentBlock("domain");
		}
	
		$tpl->setVariable("NewDomainButton","Add Domain");

		return $tpl->get();
	}

	/**
	 * 
	 */
	function updateDomain($id,$form) {
		if($this->refIAEditDomain) {
			$this->refIAEditDomain->setRootPath($this->cfg["datapath"]);
			$ret = $this->refIAEditDomain->update($form);
			
			if ($ret == 2)		// Insert
				$this->setAlert($GLOBALS['strAdminUserCreated']);
			elseif ($ret == 3)	// Insert and folders created
				$this->setAlert($GLOBALS['strAdminUserCreated']."<br>&nbsp;<br>".$GLOBALS['strImageFoldersCreated']);
			elseif ($ret == 0)
				$this->setAlert("The domain settings could not be changed. Please note that neither the domain name nor the folder name can contain special characters.");
		}	
	}	
	
	/**
	 * 
	 */
	function getLeftFrameData() {
		
		if ($this->adminDomainMode) {
			$out = "";
		} elseif ($this->refIADomain) {
			$out = $this->refIADomain->navigation();
		} else {
			$out = "Unknown Domain";
		}
		
	//	$table->addLine(array("<img src=\"images/destroy.gif\" width=\"16\" height=\"16\">"),array("destroy"=>"Destroy Session"));
		return $out;
	}
	
	/**
	 * 
	 */
	function getMainFrameData() {
	
		if ($this->alert) {
			$out = "<p class=\"alert\">".$this->alert."</p>";
			$this->alert = NULL;
		} else {
			$out = "";
		}
	
		switch($this->screen) {
			case "domainlist":
				$out .= $this->domainList();
				break;
			case "domainedit":
				$out .= $this->refIAEditDomain->domainView();
				break;
			case "notpermitted":
				$out = "<p class=\"caution\">Action not permitted!</p>";
				break;
			default:
				$out .= ($this->refIADomain) ? $this->refIADomain->content() : $this->appWelcome;
				break;
		}
		return $out;
	}

	/**
	 * 
	 */
	function getCreditsData() {
		return "<a href=\"http://imagealbum.sourceforge.net/\">".$this->appName."</a> v".$this->appVersion." - Copyright &copy; ".$this->appCopyright;
	}

	/**
	 *
	 */
	function &getTemplateInstance($name,$removeEmptyBlocks=true,$formaction=null) {
		$tplFile = "default/$name.template.html";
		$tpl = new HTML_Template_IT("./appearance");
		$tpl->loadTemplatefile($tplFile, true, $removeEmptyBlocks);

		if ($formaction) {
			$tpl->setVariable("PHP_SELF",$PHP_SELF);
			$tpl->setVariable("ACTION",$formaction);
		}

		return $tpl;
	}

}

?>
